import React, { useCallback, useMemo, useState } from 'react'
import { i18n } from '@freesewing/collection'
import {
  designOptionType,
  round,
  measurementAsUnits,
  measurementAsMm,
  formatMm,
  formatFraction128,
} from '@freesewing/utils'
import { menuRoundPct } from '../../lib/index.mjs'
import { ButtonFrame, NumberInput } from '@freesewing/react/components/Input'
import { defaultConfig } from '../../config/index.mjs'
import { ApplyIcon } from '@freesewing/react/components/Icon'
import { mergeOptions } from '@freesewing/core'
import { KeyVal } from '@freesewing/react/components/KeyVal'

/** A boolean version of {@see MenuListInput} that sets up the necessary configuration */
export const MenuBoolInput = (props) => <MenuListInput {...props} />

/** A placeholder for an input to handle constant values */
export const MenuConstantInput = ({
  type = 'number',
  name,
  current,
  updateHandler,
  changed,
  config,
}) => (
  <>
    <input
      type={type}
      className={`
      tw:daisy-input tw:daisy-input-bordered tw:w-full tw:text-base-content
      ${changed ? 'tw:daisy-input-secondary' : 'tw:daisy-input-accent'}
    `}
      value={changed ? current : config.dflt}
      onChange={(evt) => updateHandler([name], evt.target.value)}
    />
  </>
)

/** A {@see MenuSliderInput} to handle degree values */
export const MenuDegInput = (props) => {
  const { updateHandler } = props
  const degUpdateHandler = useCallback(
    (path, newVal) => {
      updateHandler(path, newVal === undefined ? undefined : Number(newVal))
    },
    [updateHandler]
  )
  return (
    <MenuSliderInput {...props} suffix="°" valFormatter={round} updateHandler={degUpdateHandler} />
  )
}

/**
 * An input for selecting and item from a list
 * @param  {String}  options.name       the name of the property this input changes
 * @param  {Object}  options.config     configuration for the input
 * @param  {String|Number}  options.current    the current value of the input
 * @param  {Function}  options.updateFunc the function called by the event handler to update the value
 * @param  {Boolean} options.compact    include descriptions with the list items?
 * @param  {Function}  options.t          translation function
 * @param  {String}  design  name of the design
 * @param  {Boolean} isDesignOption  Whether or not it's a design option
 */
export const MenuListInput = ({
  name,
  config,
  current,
  updateHandler,
  compact = false,
  changed,
  design,
  isDesignOption = false,
}) => {
  const handleChange = useSharedHandlers({
    dflt: config.dflt,
    updateHandler,
    name,
    isDesignOption,
  })

  return config.list.map((entry) => {
    const { title = false, about = false } = config
    if (!title || !about) console.log('No title or about in', { name, config, design })
    const sideBySide = config.sideBySide || about.length + title.length < 42

    return (
      <ButtonFrame
        dense={config.dense || false}
        key={entry}
        active={
          changed
            ? Array.isArray(current)
              ? current.includes(entry)
              : current === entry
            : entry === config.dflt
        }
        onClick={() => handleChange(entry)}
      >
        <div
          className={`tw:w-full tw:flex ${
            sideBySide
              ? 'tw:flex-row tw:justify-between tw:gap-2 tw:items-center'
              : 'tw:flex-col tw:items-start'
          }`}
        >
          <div className="tw:font-semibold">
            {config.doNotTranslate ? entry : config.choiceTitles[entry]}
          </div>
          {compact || !config.choiceDescriptions ? null : (
            <div
              className={`${config.dense ? 'tw:text-sm tw:leading-5 tw:py-1' : 'tw:text-base'} tw:font-normal`}
            >
              {config.choiceDescriptions[entry]}
            </div>
          )}
        </div>
      </ButtonFrame>
    )
  })
}

/** a toggle input for list/boolean values */
export const MenuListToggle = ({ config, changed, updateHandler, name }) => {
  const boolConfig = useBoolConfig(name, config)
  const handleChange = useSharedHandlers({ dflt: boolConfig.dflt, updateHandler, name })

  const dfltIndex = boolConfig.list.indexOf(boolConfig.dflt)

  const doToggle = () =>
    handleChange(boolConfig.list[changed ? dfltIndex : Math.abs(dfltIndex - 1)])

  const checked = boolConfig.dflt == false ? changed : !changed

  return (
    <input
      type="checkbox"
      className={`tw:daisy-toggle ${changed ? 'tw:daisy-toggle-accent' : 'tw:daisy-toggle-secondary'}`}
      checked={checked}
      onChange={doToggle}
      onClick={(evt) => evt.stopPropagation()}
    />
  )
}

export const MenuMmInput = (props) => {
  const { updateHandler, current, config } = props
  const units = props.settings?.units
  const imperial = units === 'imperial'
  const mmUpdateHandler = useCallback(
    (path, newCurrent) => {
      console.log('mmUpdateHandler', { path, newCurrent })
      const calcCurrent =
        typeof newCurrent === 'undefined' ? undefined : measurementAsMm(newCurrent, units)
      updateHandler(path, calcCurrent)
    },
    [updateHandler, units]
  )

  /*
   * Set a default step that matches the unit.
   *
   * Imperial:
   *   mm, inch, decimal digits precision needed:
   *   1.27 mm, 0.1 in, 2 digits
   *   3.175 mm, 1/8 in, 3 digits
   *   1.5875 mm, 1/16 in, 4 digits
   *   0.79375 mm, 1/32 in, 5 digits
   * We previously rounded mm values to 2 digits in the options and used
   *   a 1.27 mm (1/10 in) step because precision didn't support 1/16 in
   *   or 1/32 in steps.
   * We now round mm values to 4 digits and use a 1.5875 mm (1/16 in.) step.
   */
  const defaultStep = imperial ? 1.5875 : 1 // mm

  return (
    <MenuSliderInput
      {...props}
      {...{
        config: {
          ...config,
          min: measurementAsUnits(config.min, units),
          max: measurementAsUnits(config.max, units),
          dflt: measurementAsUnits(config.dflt, units),
          step: measurementAsUnits(defaultStep, units),
        },
        current: current === undefined ? undefined : measurementAsUnits(current, units),
        updateHandler: mmUpdateHandler,
        valFormatter: (val) => (imperial ? formatFraction128(val, null) : val),
        suffix: imperial ? '"' : 'cm',
      }}
    />
  )
}

/** A {@see SliderInput} to handle percentage values */
export const MenuPctInput = ({ current, changed, updateHandler, config, ...rest }) => {
  const factor = 100
  let pctCurrent = menuRoundPct(changed ? current : config.dflt, factor)
  const pctUpdateHandler = useCallback(
    (path, newVal) =>
      updateHandler(path, newVal === undefined ? undefined : menuRoundPct(newVal, 1 / factor)),
    [updateHandler]
  )

  return (
    <MenuSliderInput
      {...{
        ...rest,
        config: { ...config, dflt: menuRoundPct(config.dflt, factor) },
        current: pctCurrent,
        updateHandler: pctUpdateHandler,
        suffix: '%',
        valFormatter: round,
        changed,
      }}
    />
  )
}

/**
 * An input component that uses a slider to change a number value
 * @param  {String}   options.name         the name of the property being changed by the input
 * @param  {Object}   options.config       configuration for the input
 * @param  {Number}   options.current      the current value of the input
 * @param  {Function}   options.updateHandler   the function called by the event handler to update the value
 * @param  {Function}   options.t            translation function
 * @param  {Boolean}   options.override     open the text input to allow override of the slider?
 * @param  {String}   options.suffix       a suffix to append to value labels
 * @param  {Function} options.valFormatter a function that accepts a value and formats it for display as a label
 * @param  {Function}   options.setReset     a setter for the reset function on the parent component
 */
export const MenuSliderInput = ({
  name,
  config,
  current,
  updateHandler,
  override,
  suffix = '',
  valFormatter = (val) => val,
  setReset,
  children,
  changed,
  state,
  Design,
}) => {
  const { settings = {} } = state // Guard against undefined settings
  const { max, min } = config
  const handleChange = useSharedHandlers({
    current,
    dflt: config.dflt,
    updateHandler,
    name,
    setReset,
  })

  const val = typeof current === 'undefined' ? config.dflt : current

  if (override)
    return (
      <>
        <div className="tw:flex tw:flex-row tw:justify-between">
          <MenuEditOption
            {...{
              config,
              current: val,
              handleChange,
              min,
              max,
              state,
            }}
          />
        </div>
        {children}
      </>
    )

  return (
    <>
      <div className="tw:flex tw:flex-row tw:justify-between">
        <span className="tw:opacity-50">
          <span dangerouslySetInnerHTML={{ __html: valFormatter(min) + suffix }} />
        </span>
        <div
          className={`tw:font-bold ${val === config.dflt ? 'tw:text-secondary' : 'tw:text-accent'}`}
        >
          <span dangerouslySetInnerHTML={{ __html: valFormatter(val) + suffix }} />
          {typeof config.toAbs === 'function' ? (
            <span>
              <span className="tw:px-2">|</span>
              <span
                dangerouslySetInnerHTML={{
                  __html: formatMm(
                    config.toAbs(
                      val / 100,
                      settings,
                      mergeOptions(settings, Design.patternConfig.options)
                    ),
                    settings.units
                  ),
                }}
              />
            </span>
          ) : null}
        </div>
        <span className="tw:opacity-50">
          <span dangerouslySetInnerHTML={{ __html: valFormatter(max) + suffix }} />
        </span>
      </div>
      <input
        type="range"
        {...{ min, max, value: val, step: config.step || 0.1 }}
        onChange={(evt) => handleChange(evt.target.value)}
        className={`
          tw:daisy-range tw:daisy-range-sm tw:mt-1 tw:w-full
          ${changed ? 'tw:daisy-range-accent' : 'tw:daisy-range-secondary'}
        `}
      />
      {children}
    </>
  )
}

export const MenuEditOption = (props) => {
  const { config, handleChange } = props
  const { settings = {} } = props.state // Guard against undefined settings
  const type = designOptionType(config)

  const [manualEdit, setManualEdit] = useState(props.current)
  const [abs, setAbs] = useState(false)
  const [units, setUnits] = useState(
    abs ? (settings.units === 'imperial' ? 'inch' : 'cm') : defaultConfig.menuOptionEditLabels[type]
  )

  const onUpdate = useCallback(
    (validVal, units) => {
      if (validVal !== null && validVal !== false) {
        if (type === 'pct' && units === 'cm')
          return handleChange(config.fromAbs(Number(validVal) * 1000, settings))
        if (type === 'pct' && units === 'inch')
          return handleChange(config.fromAbs(Number(validVal) * 2540, settings))
        return handleChange(validVal)
      }
    },
    [handleChange]
  )

  const toggleInputUnits = () => {
    if (abs) setUnits(defaultConfig.menuOptionEditLabels[type])
    else setUnits(settings.units === 'imperial' ? 'inch' : 'cm')
    setAbs(!abs)
    console.log('in toogg;e')
  }

  if (!['pct', 'count', 'deg', 'mm'].includes(type))
    return <p>This design option type does not have a component to handle manual input.</p>

  return (
    <div className="tw:daisy-form-control tw:mb-2 tw:w-full">
      <div className="tw:daisy-label tw:font-medium tw:text-accent">
        <label className="tw:daisy-label-text">
          <em>Enter a custom value</em>
        </label>
        {type === 'pct' && typeof config.fromAbs === 'function' ? (
          <label className="tw:daisy-label-text">
            <KeyVal k="units" val={units} onClick={toggleInputUnits} color="secondary" />
          </label>
        ) : null}
      </div>
      <label className="tw:daisy-input-group tw:daisy-input-group-sm tw:flex tw:flex-row tw:items-end tw:gap-2 tw:-mt-4">
        <NumberInput value={manualEdit} update={setManualEdit} />
        <button
          className="tw:daisy-btn tw:daisy-btn-secondary tw:mt-4"
          onClick={() => onUpdate(manualEdit, units)}
        >
          <ApplyIcon />
        </button>
      </label>
    </div>
  )
}

/**
 * A hook to get the change handler for an input.
 * @param  {Number|String|Boolean} options.dflt       the default value for the input
 * @param  {Function}              options.updateHandler the onChange
 * @param  {string}                options.name       the name of the property being changed
 * @return the change handler for the input
 */
const useSharedHandlers = ({ dflt, updateHandler, name, isDesignOption }) => {
  return useCallback(
    (newCurrent = '__UNSET__') => {
      /*
       * When a design option is set to the default, just unset it instead
       * This is both more efficient, and  makes it easy to see in which ways
       * your pattern differs from the defaults
       */
      if (isDesignOption && newCurrent === dflt) newCurrent = '__UNSET__'
      updateHandler([name], newCurrent)
    },
    [dflt, updateHandler, name, isDesignOption]
  )
}

/** get the configuration that allows a boolean value to use the list input */
const useBoolConfig = (name, config) => {
  return useMemo(
    () => ({
      list: [false, true],
      choiceTitles: {
        false: `${name}No`,
        true: `${name}Yes`,
      },
      valueTitles: {
        false: 'no',
        true: 'yes',
      },
      ...config,
    }),
    [name, config]
  )
}

/** an input for the 'only' setting. toggles individual parts*/
export const MenuOnlySettingInput = (props) => {
  const { config } = props
  config.list = config.list.sort()
  config.choiceTitles = {}
  for (const part of config.list) {
    const [design, name] = part.split('.')
    config.choiceTitles[part] = (
      <span>
        <span className="tw:font-medium tw:opacity-80 tw:capitalize">{design}</span>
        <span className="tw:font-medium tw:opacity-80 tw:capitalize tw:px-2">&raquo;</span>
        {i18n[design].en.p[name]}
      </span>
    )
  }

  return <MenuListInput {...props} />
}

export const MenuUxSettingInput = (props) => {
  const { state, update } = props

  return <MenuListInput {...props} updateHandler={update.ui} current={state.ui.ux} />
}
