import React from 'react'
import { LineDrawingWrapper, thin, dashed } from './shared.mjs'

/*
 * This strokeScale factor is used to normalize the stroke across
 * designs so we have a consistent look when showing our collection
 */
const strokeScale = 0.5

/**
 * A linedrawing component for Carlton
 *
 * @component
 * @param {object} props - All component props
 * @param {string} props.className - Any CSS classes to apply
 * @param {number} props.stroke - The stroke width to apply
 * @returns {JSX.Element}
 */
export const Carlton = ({ className, stroke = 1 }) => (
  <LineDrawingWrapper viewBox="0 0 194 175" {...{ className, stroke }}>
    <Front stroke={stroke * strokeScale} />
    <Back stroke={stroke * strokeScale} />
  </LineDrawingWrapper>
)

/**
 * A linedrawing component for the front of Carlton
 *
 * @component
 * @param {object} props - All component props
 * @param {string} props.className - Any CSS classes to apply
 * @param {number} props.stroke - The stroke width to apply
 * @returns {JSX.Element}
 */
export const CarltonFront = ({ className, stroke = 1 }) => (
  <LineDrawingWrapper viewBox="-35 4 168 168" {...{ className }}>
    <Front stroke={stroke * strokeScale} />
  </LineDrawingWrapper>
)

/**
 * A linedrawing component for the back of Carlton
 *
 * @component
 * @param {object} props - All component props
 * @param {string} props.className - Any CSS classes to apply
 * @param {number} props.stroke - The stroke width to apply
 * @returns {JSX.Element}
 */
export const CarltonBack = ({
  className = 'w-64', // CSS classes to apply
  stroke = 1, // Stroke width to use
}) => {
  // Normalize stroke across designs
  stroke = stroke * strokeScale

  return (
    <LineDrawingWrapper viewBox="97 0 97 175" {...{ className, stroke }}>
      <Back stroke={stroke} />
    </LineDrawingWrapper>
  )
}

/*
 * SVG elements for the front
 */
const Front = ({ stroke }) => (
  <>
    <path
      key="stitches"
      {...dashed(stroke)}
      {...thin(stroke)}
      d="m 69.71,83.03 -5.63,0.5 1.27,5.83

      m 0.19,0 4.78,-0.5

      m -46.66,-5.83 5.63,0.5 -1.27,5.83

      m -0.19,0 -4.79,-0.5

      m 64.78,6.2
      c -0.78,0.31 -1.62,0.59 -2.53,0.84 -3.23,0.89 -6.15,0.99 -8.44,0.84
      l -0.26,0.13

      m -1.01,67.83
      c -0.95,0.35 -2.41,0.85 -4.22,1.27 -2.43,0.56 -4.38,0.74 -5.91,0.84 -6.67,0.46 -17.53,0.43 -18.15,0.42
      h -1.27
      c -0.63,0 -11.48,0.04 -18.15,-0.42 -1.53,-0.1 -3.48,-0.28 -5.91,-0.84 -1.82,-0.42 -3.27,-0.92 -4.22,-1.27

      m -1.43,-67.83 -0.26,-0.13
      c -2.29,0.15 -5.22,0.04 -8.44,-0.84 -0.91,-0.25 -1.75,-0.54 -2.53,-0.84"
    />
    <path
      key="folds"
      opacity={0.3}
      d="m 67.7,42.51
      c 0.11,0.61 0.3,1.21 0.46,1.81
      l 0.52,1.79 1.06,3.58
      c 0.7,2.39 1.36,4.79 1.97,7.21 1.22,4.83 2.24,9.71 3.1,14.62 0.83,4.91 1.51,9.85 2.1,14.79 0.28,2.48 0.51,4.95 0.74,7.43 0.21,2.48 0.41,4.96 0.61,7.44 -0.24,-2.48 -0.49,-4.96 -0.75,-7.43 -0.26,-2.48 -0.52,-4.95 -0.82,-7.42 -0.64,-4.93 -1.31,-9.87 -2.17,-14.77 -0.83,-4.91 -1.86,-9.78 -3.03,-14.61 -0.58,-2.42 -1.21,-4.83 -1.88,-7.22
      l -0.99,-3.6 -0.48,-1.81
      c -0.15,-0.6 -0.33,-1.2 -0.43,-1.82
      z

      m -42.21,0
      c -0.1,0.61 -0.28,1.21 -0.43,1.82
      l -0.48,1.81 -0.99,3.6
      c -0.67,2.39 -1.3,4.8 -1.88,7.22 -1.17,4.84 -2.2,9.71 -3.03,14.61 -0.86,4.9 -1.53,9.83 -2.17,14.77 -0.3,2.47 -0.57,4.94 -0.82,7.42 -0.26,2.48 -0.51,4.95 -0.75,7.43 0.19,-2.48 0.39,-4.96 0.61,-7.44 0.23,-2.48 0.46,-4.96 0.74,-7.43 0.59,-4.94 1.27,-9.88 2.1,-14.79 0.86,-4.91 1.88,-9.79 3.1,-14.62 0.61,-2.42 1.26,-4.82 1.97,-7.21
      l 1.06,-3.58 0.52,-1.79
      c 0.16,-0.6 0.35,-1.2 0.46,-1.81
      z

      m 30.18,18.79
      c 0,0.7 -0.57,1.27 -1.27,1.27 -0.7,0 -1.27,-0.57 -1.27,-1.27 0,-0.7 0.57,-1.27 1.27,-1.27 0.7,0 1.27,0.57 1.27,1.27
      z

      m 0,22.37
      c 0,0.7 -0.57,1.27 -1.27,1.27 -0.7,0 -1.27,-0.57 -1.27,-1.27 0,-0.7 0.57,-1.27 1.27,-1.27 0.7,0 1.27,0.57 1.27,1.27
      z

      m 0,-10.97
      c 0,0.7 -0.57,1.27 -1.27,1.27 -0.7,0 -1.27,-0.57 -1.27,-1.27 0,-0.7 0.57,-1.27 1.27,-1.27 0.7,0 1.27,0.57 1.27,1.27
      z

      m -12.24,0
      c 0,0.7 -0.57,1.27 -1.27,1.27 -0.7,0 -1.27,-0.57 -1.27,-1.27 0,-0.7 0.57,-1.27 1.27,-1.27 0.7,0 1.27,0.57 1.27,1.27
      z

      m 0,10.97
      c 0,0.7 -0.57,1.27 -1.27,1.27 -0.7,0 -1.27,-0.57 -1.27,-1.27 0,-0.7 0.57,-1.27 1.27,-1.27 0.7,0 1.27,0.57 1.27,1.27
      z

      m 0,-21.95
      c 0,0.7 -0.57,1.27 -1.27,1.27 -0.7,0 -1.27,-0.57 -1.27,-1.27 0,-0.7 0.57,-1.27 1.27,-1.27 0.7,0 1.27,0.57 1.27,1.27
      z

      m 28.11,-45.73
      c -0.55,2.52 -1.02,4.37 -1.5,6.9 -0.49,2.53 -0.92,5.07 -1.27,7.62 -0.38,2.55 -0.67,5.11 -0.92,7.67 -0.12,1.28 -0.23,2.56 -0.32,3.85 -0.1,1.28 -0.17,2.57 -0.25,3.86 0.01,-1.29 0.05,-2.58 0.1,-3.87 0.06,-1.29 0.14,-2.57 0.24,-3.86 0.2,-2.57 0.49,-5.14 0.85,-7.69 0.38,-2.55 0.81,-5.1 1.35,-7.62 0.27,-1.26 0.55,-2.52 0.87,-3.77
      z

      m -50.01,-0.65 0.99,3.74
      c 0.32,1.25 0.59,2.51 0.87,3.77 0.54,2.52 0.97,5.07 1.35,7.62 0.35,2.56 0.65,5.12 0.85,7.69 0.09,1.29 0.18,2.57 0.24,3.86 0.05,1.29 0.09,2.58 0.1,3.87 -0.08,-1.29 -0.15,-2.57 -0.25,-3.86 -0.09,-1.28 -0.2,-2.57 -0.32,-3.85 -0.25,-2.56 -0.54,-5.12 -0.92,-7.67 -0.35,-2.55 -0.79,-5.09 -1.27,-7.62
      z

      m 45.97,40.89
      c -0.64,1.85 -1.23,3.71 -1.77,5.58 -0.53,1.88 -1.03,3.76 -1.44,5.67 -0.43,1.9 -0.79,3.82 -1.09,5.75 -0.31,1.93 -0.55,3.86 -0.76,5.8 0.04,-0.97 0.12,-1.95 0.2,-2.92 0.09,-0.97 0.21,-1.94 0.34,-2.91 0.26,-1.94 0.62,-3.86 1.03,-5.78 0.43,-1.91 0.93,-3.8 1.51,-5.67 0.29,-0.93 0.6,-1.86 0.93,-2.78 0.34,-0.92 0.68,-1.83 1.06,-2.73
      z

      m -41.79,0
      c 0.38,0.9 0.72,1.81 1.06,2.73 0.33,0.92 0.64,1.85 0.93,2.78 0.58,1.87 1.08,3.76 1.51,5.67 0.41,1.91 0.77,3.84 1.03,5.78 0.13,0.97 0.24,1.94 0.34,2.91 0.08,0.97 0.16,1.95 0.2,2.92 -0.21,-1.94 -0.46,-3.88 -0.76,-5.8 -0.31,-1.93 -0.66,-3.84 -1.09,-5.75 -0.41,-1.91 -0.91,-3.79 -1.44,-5.67 -0.54,-1.88 -1.13,-3.73 -1.77,-5.58
      z"
    />
    <path
      key="outline"
      d="m 64.15,86.25 0.28,9.57
      c 0.01,0.39 0.01,1.09 0,1.96 -0.06,5.16 -0.1,7.05 -0.1,7.05 -0.01,0.32 -0.02,1.12 0.2,2.09 0.09,0.4 0.23,0.98 0.61,1.57 0.07,0.11 0.41,0.64 1.01,1.04 0,0 0.6,0.4 1.41,0.52 0.65,0.1 2.54,0.03 5.27,-0.21

      m -3.05,-27.6 -6.47,0.65 1.45,7.57 6.05,-0.65

      m -41.41,-3.56 -0.28,9.57
      c -0.01,0.39 -0.01,1.09 0,1.96 0.06,5.16 0.1,7.05 0.1,7.05 0.01,0.32 0.02,1.12 -0.2,2.09 -0.09,0.4 -0.23,0.98 -0.61,1.57 -0.07,0.11 -0.41,0.64 -1.01,1.04 0,0 -0.6,0.4 -1.41,0.52 -0.65,0.1 -2.54,0.03 -5.27,-0.21

      m 3.05,-27.6 6.46,0.65 -1.45,7.57 -6.04,-0.65

      m 64.98,4.5
      c -0.78,0.31 -1.62,0.59 -2.53,0.84 -3.22,0.88 -6.14,0.99 -8.43,0.84
      l -0.26,0.13

      m -59.76,0 -0.26,-0.13
      c -2.29,0.15 -5.21,0.04 -8.43,-0.84
      C 7.14,94.91 6.3,94.62 5.52,94.31

      m 52.46,-83.94
      c 0,0 15.38,6.07 15.9,6.66 1.1,1.26 1.74,2.5 2.11,3.38 0.43,0.99 1.05,2.44 1.69,4.22 0.13,0.36 1.25,3.52 2.11,7.18 2.17,9.26 4.22,26.17 4.22,26.17 1.58,13.07 2.39,19.73 2.95,25.75 0.58,6.16 0.98,11.47 1.26,15.62 -0.78,0.31 -1.62,0.59 -2.53,0.84 -3.22,0.88 -6.14,0.99 -8.43,0.84
      l -0.26,0.13
      C 76.77,98.84 76.56,97.76 76.41,96.38 75.66,89.69 73.74,74.12 69.24,54.17 68.53,50.99 67.91,48.44 67.55,46.99
      c 0.17,4.98 0.52,13.13 1.26,23.21 1.3,17.63 2.5,23.28 4.22,44.32 1.06,13.03 1.54,22.45 1.69,25.33 0.53,10.8 0.75,20 0.84,27.02 -0.95,0.35 -2.4,0.85 -4.22,1.27 -2.43,0.56 -4.38,0.74 -5.9,0.84 -6.66,0.46 -17.5,0.43 -18.13,0.42
      h -1.26
      c -0.63,0 -11.47,0.04 -18.13,-0.42 -1.52,-0.1 -3.48,-0.28 -5.9,-0.84 -1.82,-0.42 -3.27,-0.91 -4.22,-1.27 0.09,-7.01 0.31,-16.22 0.84,-27.02 0.14,-2.87 0.63,-12.3 1.69,-25.33 1.71,-21.04 2.91,-26.69 4.22,-44.32 0.75,-10.08 1.09,-18.24 1.26,-23.21 -0.36,1.45 -0.97,4 -1.69,7.18 -4.5,19.95 -6.42,35.52 -7.17,42.21 -0.16,1.39 -0.36,2.46 -0.58,4.78
      l -0.26,-0.13
      c -2.29,0.15 -5.21,0.04 -8.43,-0.84
      C 6.78,99.93 5.93,99.64 5.16,99.33 5.44,95.19 5.85,89.88 6.42,83.72 6.98,77.7 7.79,71.04 9.37,57.97
      c 0,0 2.05,-16.91 4.22,-26.17 0.86,-3.66 1.98,-6.82 2.11,-7.18 0.64,-1.78 1.25,-3.24 1.69,-4.22 0.37,-0.88 1.01,-2.12 2.11,-3.38 0.51,-0.59 15.9,-6.66 15.9,-6.66

      m 17.6,0.33
      h -5.91 -0.58 -5.91

      m 17.6,5.86 1.77,1.95
      c -6.07,15.44 -7.99,22.1 -20.63,38.2
      L 38.47,169.41

      m 14.52,-158.71 5.19,5.84
      c 1.46,-0.39 2.82,-0.6 4.93,-0.77
      L 53.83,5.63
      H 47.07 46.48 39.72
      L 30.45,15.76
      c 1.91,0.27 4.29,0.53 5.14,0.48
      l 4.98,-5.55

      M 53.19,6.48
      c 0.08,1.71 0.05,3.43 -0.1,5.13 -0.08,0.85 -0.19,1.7 -0.31,2.55 -0.12,0.85 -0.25,1.69 -0.39,2.54 -0.28,1.69 -0.58,3.37 -0.9,5.05 -0.17,0.84 -0.34,1.68 -0.52,2.51
      l -0.28,1.25 -0.15,0.63 -0.07,0.31 -0.04,0.16
      c -0.01,0.05 -0.02,0.11 -0.04,0.12
      l -0.01,0.04
      c -0.7,2.57 -1.45,5.13 -2.27,7.66 -0.82,2.53 -1.69,5.05 -2.61,7.55 -0.93,2.5 -1.91,4.97 -2.95,7.42 -1.03,2.45 -2.15,4.87 -3.29,7.27 1.09,-2.43 2.13,-4.87 3.13,-7.34 1.01,-2.46 1.95,-4.95 2.86,-7.45 0.91,-2.5 1.77,-5.02 2.58,-7.55 0.8,-2.53 1.57,-5.08 2.28,-7.64
      l -0.01,0.04 0.31,-1.28 0.28,-1.25
      c 0.19,-0.83 0.36,-1.67 0.53,-2.51 0.34,-1.67 0.67,-3.35 0.96,-5.04 0.15,-0.84 0.29,-1.69 0.42,-2.53 0.14,-0.84 0.27,-1.69 0.36,-2.54 0.19,-1.7 0.26,-3.41 0.22,-5.12
      z

      m -12.62,0
      c -0.03,1.71 0.05,3.42 0.24,5.12 0.19,1.69 0.52,3.38 0.79,5.06 0.29,1.69 0.61,3.37 0.95,5.04
      l 0.52,2.51 0.28,1.25 0.3,1.28
      v -0.03
      c 0.47,1.85 0.96,3.7 1.51,5.53 0.55,1.83 1.12,3.66 1.73,5.47
      C 46.21,35.92 45.57,34.12 44.99,32.29 44.4,30.47 43.87,28.62 43.39,26.77
      v -0.03
      c 0,0 -0.02,-0.07 -0.04,-0.12
      l -0.04,-0.16 -0.08,-0.31 -0.15,-0.62 -0.29,-1.25
      C 42.61,23.43 42.43,22.6 42.27,21.76 41.93,20.08 41.63,18.39 41.35,16.7 41.2,15.86 41.08,15.01 40.96,14.16 40.83,13.32 40.72,12.47 40.64,11.61 40.49,9.9 40.46,8.18 40.57,6.48
      Z

      m -4.98,9.79 -2.13,2.38
      c 4.58,10.08 6.64,13.65 11.11,24.85"
    />
  </>
)

/*
 * SVG elements for the back
 */
const Back = ({ stroke }) => (
  <>
    <path
      key="outline"
      d="m 147.24,5.15
      h 7.41
      c 0,0 4.64,5.07 4.64,5.07 -1.49,0.34 -3.04,0.62 -4.64,0.84 -2.53,0.34 -5.01,0.5 -7.41,0.49

      m 0.24,-6.4
      h -7.41
      l -4.64,5.07
      c 1.49,0.34 3.04,0.62 4.64,0.84 2.53,0.34 5.01,0.5 7.41,0.49

      m -14.98,52.59
      V 58.34
      h 29.76
      v 5.81
      z

      m -2.51,22.06 -0.28,9.57
      c -0.01,0.39 -0.01,1.09 0,1.96 0.06,5.16 0.1,7.05 0.1,7.05 0.01,0.32 0.02,1.12 -0.2,2.09 -0.09,0.4 -0.23,0.98 -0.61,1.57 -0.07,0.11 -0.41,0.64 -1.01,1.04 0,0 -0.6,0.4 -1.41,0.52 -0.65,0.1 -2.54,0.03 -5.27,-0.21

      m 3.05,-27.6 6.47,0.65 -1.46,7.57 -6.05,-0.65

      m 23.94,-78.28
      v 157.66

      m 17.52,-82.95 0.28,9.57
      c 0.01,0.39 0.01,1.09 0,1.96 -0.06,5.16 -0.1,7.05 -0.1,7.05 -0.01,0.32 -0.02,1.12 0.2,2.09 0.09,0.4 0.23,0.98 0.61,1.57 0.07,0.11 0.41,0.64 1.01,1.04 0,0 0.6,0.4 1.41,0.52 0.65,0.1 2.54,0.03 5.27,-0.21

      m -3.05,-27.6 -6.47,0.65 1.46,7.57 6.05,-0.65

      m -65.07,4.5
      c 0.78,0.31 1.62,0.59 2.53,0.84 3.23,0.89 6.15,0.99 8.44,0.84
      l 0.26,0.13

      m 59.85,0 0.26,-0.13
      c 2.29,0.15 5.22,0.04 8.44,-0.84 0.91,-0.25 1.75,-0.54 2.53,-0.84

      m -52.53,-83.95
      c 0,0 -15.4,6.07 -15.92,6.66 -1.1,1.26 -1.74,2.5 -2.11,3.38 -0.43,0.99 -1.05,2.44 -1.69,4.22 -0.13,0.36 -1.25,3.52 -2.11,7.18 -2.17,9.26 -4.22,26.17 -4.22,26.17 -1.59,13.07 -2.39,19.73 -2.96,25.75 -0.58,6.16 -0.98,11.47 -1.27,15.62 0.78,0.31 1.62,0.59 2.53,0.84 3.23,0.89 6.15,0.99 8.44,0.84
      l 0.26,0.13
      c 0.22,-2.32 0.43,-3.39 0.58,-4.78 0.75,-6.69 2.67,-22.26 7.18,-42.21 0.72,-3.18 1.33,-5.72 1.69,-7.18 -0.17,4.98 -0.52,13.13 -1.27,23.22 -1.3,17.63 -2.5,23.28 -4.22,44.33 -1.06,13.03 -1.54,22.45 -1.69,25.33 -0.53,10.8 -0.75,20 -0.84,27.02 0.95,0.35 2.41,0.85 4.22,1.27 2.43,0.56 4.38,0.74 5.91,0.84 6.67,0.46 17.53,0.43 18.15,0.42
      h 1.27
      c 0.63,0 11.48,0.04 18.15,-0.42 1.53,-0.1 3.48,-0.28 5.91,-0.84 1.82,-0.42 3.27,-0.92 4.22,-1.27 -0.09,-7.01 -0.31,-16.22 -0.84,-27.02 -0.14,-2.87 -0.63,-12.3 -1.69,-25.33 -1.72,-21.05 -2.92,-26.69 -4.22,-44.33 -0.75,-10.08 -1.09,-18.24 -1.27,-23.22 0.36,1.45 0.97,4 1.69,7.18 4.5,19.95 6.43,35.53 7.18,42.21 0.16,1.39 0.36,2.46 0.58,4.78
      l 0.26,-0.13
      c 2.29,0.15 5.22,0.04 8.44,-0.84 0.91,-0.25 1.75,-0.54 2.53,-0.84 -0.28,-4.15 -0.69,-9.46 -1.27,-15.62 -0.56,-6.02 -1.37,-12.68 -2.96,-25.75 0,0 -2.05,-16.91 -4.22,-26.17 -0.86,-3.66 -1.98,-6.82 -2.11,-7.18 -0.64,-1.78 -1.25,-3.24 -1.69,-4.22 -0.37,-0.88 -1.01,-2.12 -2.11,-3.38 -0.51,-0.59 -15.92,-6.66 -15.92,-6.66"
    />
    <path
      key="folds"
      opacity={0.3}
      d="m 159.93,61.3
      c 0,0.7 -0.57,1.27 -1.27,1.27 -0.7,0 -1.27,-0.57 -1.27,-1.27 0,-0.7 0.57,-1.27 1.27,-1.27 0.7,0 1.27,0.57 1.27,1.27
      z

      m -22.8,0
      c 0,0.7 -0.57,1.27 -1.27,1.27 -0.7,0 -1.27,-0.57 -1.27,-1.27 0,-0.7 0.57,-1.27 1.27,-1.27 0.7,0 1.27,0.57 1.27,1.27
      z

      M 159.03,65
      c 0.12,0.64 0.21,1.28 0.29,1.93 0.09,0.65 0.15,1.29 0.21,1.94 0.06,0.65 0.11,1.29 0.14,1.94
      l 0.05,0.97
      c 0.01,0.33 0.02,0.65 0.01,0.98 -0.06,-0.32 -0.11,-0.64 -0.16,-0.96
      l -0.13,-0.97
      c -0.09,-0.65 -0.15,-1.29 -0.21,-1.94 -0.05,-0.65 -0.11,-1.29 -0.14,-1.94 -0.04,-0.65 -0.07,-1.3 -0.06,-1.95
      z

      m 0.71,-18.48
      c -0.09,0.92 -0.22,1.84 -0.34,2.75 -0.06,0.46 -0.14,0.91 -0.2,1.37 -0.07,0.46 -0.13,0.91 -0.18,1.37
      l -0.69,5.49
      c 0,-0.92 0.04,-1.85 0.11,-2.77 0.06,-0.92 0.16,-1.84 0.28,-2.76 0.06,-0.46 0.12,-0.92 0.2,-1.37 0.08,-0.45 0.16,-0.91 0.26,-1.36 0.18,-0.91 0.36,-1.81 0.57,-2.71
      z

      M 135.68,65
      c 0,0.65 -0.03,1.3 -0.06,1.95 -0.03,0.65 -0.09,1.3 -0.14,1.94 -0.06,0.65 -0.13,1.29 -0.21,1.94
      l -0.13,0.97
      c -0.05,0.32 -0.1,0.64 -0.16,0.96 -0,-0.33 0.01,-0.65 0.01,-0.98
      l 0.05,-0.97
      c 0.03,-0.65 0.09,-1.3 0.14,-1.94 0.06,-0.65 0.13,-1.29 0.21,-1.94 0.08,-0.65 0.17,-1.29 0.29,-1.93
      z

      m -0.71,-18.48
      c 0.21,0.9 0.4,1.81 0.57,2.71 0.09,0.45 0.17,0.91 0.26,1.36 0.07,0.46 0.14,0.92 0.2,1.37 0.11,0.92 0.22,1.84 0.28,2.76 0.07,0.92 0.11,1.84 0.11,2.77
      l -0.69,-5.49
      c -0.06,-0.46 -0.12,-0.92 -0.18,-1.37 -0.07,-0.46 -0.14,-0.91 -0.2,-1.37 -0.13,-0.91 -0.25,-1.83 -0.34,-2.75
      z

      m -10.71,-22.58
      c -0.05,0.41 -0.16,0.8 -0.26,1.2
      l -0.3,1.19 -0.62,2.38 -1.27,4.74
      c -0.87,3.16 -1.68,6.33 -2.44,9.51 -1.51,6.37 -2.81,12.79 -3.87,19.25 -1.1,6.45 -1.97,12.94 -2.78,19.44 -0.38,3.25 -0.71,6.51 -1.02,9.77 -0.33,3.26 -0.15,0.74 -0.45,4 0.26,-3.26 0.02,-0.75 0.3,-4.01 0.29,-3.26 0.61,-6.52 0.97,-9.78 0.77,-6.5 1.62,-13 2.69,-19.46 1.1,-6.46 2.43,-12.88 3.98,-19.24 0.78,-3.18 1.6,-6.35 2.49,-9.5
      l 1.34,-4.72 0.65,-2.37 0.32,-1.19
      c 0.1,-0.4 0.21,-0.79 0.27,-1.2
      z

      m 46.86,0
      c 0.05,0.41 0.17,0.8 0.26,1.2
      l 0.3,1.19 0.61,2.37 1.27,4.73
      c 0.84,3.16 1.61,6.34 2.34,9.52 1.45,6.38 2.7,12.79 3.73,19.25 1,6.46 1.78,12.95 2.51,19.45 0.34,3.25 0.63,6.51 0.91,9.76 0.26,3.26 0.04,0.9 0.28,4.16 -0.28,-3.25 -0.58,-6.51 -0.89,-9.76 -0.3,-3.25 -0.14,-0.89 -0.49,-4.13 -0.76,-6.49 -1.58,-12.97 -2.61,-19.42 -1,-6.46 -2.21,-12.88 -3.63,-19.25 -0.71,-3.19 -1.47,-6.37 -2.29,-9.53
      l -1.2,-4.75 -0.58,-2.38 -0.28,-1.19
      c -0.09,-0.4 -0.19,-0.8 -0.24,-1.2
      z

      m -48.56,-7.99 0.83,3.13
      c 0.32,1.25 0.59,2.51 0.87,3.77 0.54,2.52 0.97,5.07 1.35,7.62 0.35,2.56 0.65,5.12 0.85,7.69 0.09,1.29 0.18,2.57 0.24,3.86 0.05,1.29 0.09,2.58 0.1,3.87 -0.08,-1.29 -0.15,-2.57 -0.25,-3.86 -0.09,-1.28 -0.2,-2.57 -0.32,-3.85 -0.25,-2.56 -0.54,-5.12 -0.92,-7.67 -0.35,-2.55 -0.79,-5.09 -1.27,-7.62
      z

      m 49.83,-0.05
      c -0.55,2.52 -0.99,4.46 -1.46,7 -0.49,2.53 -0.92,5.07 -1.27,7.62 -0.38,2.55 -0.67,5.11 -0.92,7.67 -0.12,1.28 -0.23,2.56 -0.32,3.85 -0.1,1.28 -0.17,2.57 -0.25,3.86 0.01,-1.29 0.05,-2.58 0.1,-3.87 0.06,-1.29 0.14,-2.57 0.24,-3.86 0.2,-2.57 0.49,-5.14 0.85,-7.69 0.38,-2.55 0.81,-5.1 1.35,-7.62 0.27,-1.26 0.55,-2.52 0.87,-3.77
      z"
    />
    <path
      key="stitches"
      {...dashed(stroke)}
      {...thin(stroke)}
      d="m 124.36,83.03 5.63,0.5 -1.27,5.83

      m -0.19,0 -4.79,-0.5

      m -5.26,75.86
      c 0.95,0.35 2.41,0.85 4.22,1.27 2.43,0.56 4.38,0.74 5.91,0.84 6.67,0.46 17.53,0.43 18.15,0.42
      h 1.27
      c 0.63,0 11.48,0.04 18.15,-0.42 1.53,-0.1 3.48,-0.28 5.91,-0.84 1.82,-0.42 3.27,-0.92 4.22,-1.27

      m -5.9,-81.68 -5.63,0.5 1.27,5.83

      m 0.19,0 4.79,-0.5

      m 6.72,8.03 0.26,-0.13
      c 2.29,0.15 5.22,0.04 8.44,-0.84 0.91,-0.25 1.75,-0.54 2.53,-0.84

      m -82.74,-0
      c 0.78,0.31 1.62,0.59 2.53,0.84 3.23,0.89 6.15,0.99 8.44,0.84
      l 0.26,0.13"
    />
  </>
)
