import React from 'react'
import { LineDrawingWrapper, thin, regular, dashed } from './shared.mjs'

/*
 * This strokeScale factor is used to normalize the stroke across
 * designs so we have a consistent look when showing our collection
 */
const strokeScale = 1.3

/**
 * A linedrawing component for Cornelius
 *
 * @component
 * @param {object} props - All component props
 * @param {string} props.className - Any CSS classes to apply
 * @param {number} props.stroke - The stroke width to apply
 * @returns {JSX.Element}
 */
export const Cornelius = ({ className, stroke = 1 }) => (
  <LineDrawingWrapper viewBox="0 0 320 218" {...{ className, stroke }}>
    <Front stroke={stroke * strokeScale} />
    <Back stroke={stroke * strokeScale} />
  </LineDrawingWrapper>
)

/**
 * A linedrawing component for the front of Cornelius
 *
 * @component
 * @param {object} props - All component props
 * @param {string} props.className - Any CSS classes to apply
 * @param {number} props.stroke - The stroke width to apply
 * @returns {JSX.Element}
 */
export const CorneliusFront = ({ className, stroke = 1 }) => (
  <LineDrawingWrapper viewBox="-30 0 215 215" {...{ className, stroke }}>
    <Front stroke={stroke * strokeScale} />
  </LineDrawingWrapper>
)

/**
 * A linedrawing component for the back of Cornelius
 *
 * @component
 * @param {object} props - All component props
 * @param {string} props.className - Any CSS classes to apply
 * @param {number} props.stroke - The stroke width to apply
 * @returns {JSX.Element}
 */
export const CorneliusBack = ({
  className = 'w-64', // CSS classes to apply
  stroke = 1, // Stroke width to use
}) => {
  // Normalize stroke across designs
  stroke = stroke * strokeScale

  return (
    <LineDrawingWrapper viewBox="160 0 160 218" {...{ className, stroke }}>
      <Back stroke={stroke} />
    </LineDrawingWrapper>
  )
}

/*
 * SVG elements for the front
 */
const Front = ({ stroke }) => (
  <>
    <path
      key="stitches"
      {...dashed(stroke)}
      {...thin(stroke)}
      d="M 124.08,41.14
      C 112.73,36.78 108.77,35.11 101.84,30.75 97.93,28.29 95.16,26.59 91.11,21.94

      M 28.11,41.31
      c 11.35,-4.35 15.31,-6.03 22.23,-10.38 3.92,-2.47 6.68,-4.16 10.74,-8.81

      m 22.65,0.09
      c 0,0 0.5,39.87 0,41.45 -0.89,2.83 -3.74,4.94 -7.32,5.52"
    />
    <path
      key="folds"
      opacity={0.3}
      {...regular(stroke)}
      d="m 126.02,106.21
      c 0.78,3.89 1.46,7.81 2,11.75 0.53,3.94 0.93,7.89 1.21,11.86 0.49,7.93 0.81,15.89 0.44,23.84 -0.41,7.94 -1.58,15.86 -3.55,23.56 -0.97,3.85 -2.16,7.66 -3.53,11.39 -1.39,3.72 -2.96,7.38 -4.71,10.95 1.7,-3.59 3.21,-7.27 4.54,-11.01 1.34,-3.73 2.51,-7.53 3.46,-11.39 1.93,-7.7 3.05,-15.59 3.42,-23.52 0.42,-7.92 0.14,-15.88 -0.31,-23.81 -0.26,-3.96 -0.64,-7.91 -1.15,-11.85 -0.48,-3.94 -1.1,-7.87 -1.82,-11.77
      z

      m -34.81,90.44 -1.48,-8.16
      c -0.22,-1.36 -0.37,-2.74 -0.56,-4.11
      l -0.53,-4.11 0.17,0.05
      c -0.94,1.34 -1.41,2.97 -1.53,4.61 -0.15,1.64 -0.03,3.29 0.25,4.91 0.28,1.62 0.68,3.23 1.2,4.8 0.51,1.57 1.11,3.11 1.78,4.63
      l -0.03,0.01
      c -0.74,-1.48 -1.38,-3.01 -1.92,-4.59 -0.54,-1.57 -1.01,-3.17 -1.28,-4.81 -0.28,-1.64 -0.41,-3.31 -0.24,-4.98 0.15,-1.66 0.64,-3.32 1.62,-4.69
      l 0.14,-0.2 0.03,0.24 0.5,4.11
      c 0.18,1.37 0.31,2.74 0.52,4.11
      l 1.39,8.17
      h -0.03
      z

      m 10.5,1.35
      c -0.75,-3.01 -1.31,-6.07 -1.7,-9.15 -0.39,-3.08 -0.61,-6.18 -0.57,-9.29 0.05,-3.1 0.35,-6.22 1.1,-9.24 0.75,-3.01 1.99,-5.95 3.98,-8.37
      l -0.02,0.09 -0.03,-0.14 -0.02,-0.07
      v -0.02
      c 0,0 0,-0 0,-0 -0.12,0.09 0.31,0 0.2,-0.04
      v 0.03
      c 0,0 0.06,0.28 0.06,0.28
      l 0.11,0.56 0.22,1.11 0.45,2.22
      c 0.15,0.74 0.31,1.48 0.5,2.2
      l 0.55,2.19
      c 0.75,2.93 1.36,5.89 1.87,8.87 0.25,1.49 0.45,3 0.5,4.51 0.05,1.51 0,3.03 -0.14,4.54 -0.27,3.01 -0.89,5.99 -1.73,8.89
      h -0.03
      c 0.77,-2.92 1.35,-5.9 1.59,-8.9 0.13,-1.5 0.15,-3.01 0.09,-4.52 -0.07,-1.51 -0.28,-2.99 -0.54,-4.48 -0.52,-2.97 -1.12,-5.93 -1.87,-8.85
      L 105.75,168.22
      c -0.19,-0.73 -0.35,-1.47 -0.49,-2.21
      l -0.44,-2.22 -0.22,-1.11 -0.11,-0.56 -0.05,-0.28
      v -0.03
      c -0.12,-0.05 0.31,-0.14 0.2,-0.05
      v 0.01
      c 0,0 0,0.02 0,0.02
      l 0.02,0.07 0.03,0.14 0.01,0.05 -0.03,0.03
      c -1.98,2.38 -3.22,5.29 -3.98,8.28 -0.76,3 -1.07,6.1 -1.13,9.2 -0.06,3.1 0.14,6.2 0.52,9.28 0.38,3.08 0.92,6.14 1.65,9.15
      h -0.03
      z

      m 20.3,-35.69
      c 0.45,0.67 0.79,1.4 1.05,2.16 0.24,0.77 0.42,1.55 0.51,2.35 0.19,1.6 0.1,3.21 -0.12,4.8 -0.46,3.18 -1.44,6.25 -2.51,9.27 -1.1,3.01 -2.24,6 -3.54,8.94 -1.34,2.91 -2.79,5.77 -4.47,8.51
      l -0.02,-0.02
      c 1.54,-2.8 2.99,-5.66 4.26,-8.6 0.65,-1.46 1.27,-2.94 1.85,-4.43 0.59,-1.49 1.19,-2.97 1.74,-4.48 1.1,-3 2.1,-6.06 2.58,-9.22 0.24,-1.58 0.34,-3.18 0.17,-4.77 -0.09,-0.8 -0.26,-1.58 -0.48,-2.34 -0.25,-0.76 -0.59,-1.49 -1.02,-2.16
      l 0.02,-0.02
      z

      m -2.01,-2.88
      c -0.82,-0.03 -1.63,0.28 -2.24,0.81 -0.63,0.52 -1.11,1.2 -1.5,1.91 -0.78,1.44 -1.26,3.01 -1.65,4.6 -0.77,3.18 -1.18,6.43 -1.54,9.69 -0.34,3.25 -0.68,6.51 -0.87,9.78 -0.23,3.26 -0.32,6.54 -0.36,9.81
      h -0.03
      c -0.11,-3.27 -0.02,-6.55 0.14,-9.83 0.1,-1.64 0.23,-3.27 0.38,-4.9 0.17,-1.63 0.35,-3.25 0.54,-4.89 0.39,-3.25 0.82,-6.51 1.62,-9.69 0.41,-1.58 0.91,-3.16 1.71,-4.61 0.4,-0.72 0.89,-1.4 1.53,-1.92 0.63,-0.53 1.45,-0.84 2.28,-0.8
      v 0.03
      z

      m -27.36,18.68
      c -0.16,3.38 0.09,6.78 0.75,10.1 0.66,3.31 1.74,6.55 3.34,9.53
      l -0.2,0.05 -0.28,-4.8
      c -0.05,-1.6 -0.25,-3.19 -0.42,-4.79 -0.34,-3.19 -0.86,-6.35 -1.47,-9.5
      h 0.03
      c 0.75,3.12 1.28,6.29 1.69,9.48 0.09,0.8 0.18,1.6 0.26,2.4
      l 0.11,1.2 0.07,1.2 0.23,4.81 0.02,0.46 -0.22,-0.41
      c -1.58,-3.01 -2.65,-6.27 -3.28,-9.6 -0.63,-3.33 -0.84,-6.74 -0.65,-10.12
      z

      M 26.15,106.21
      c -0.72,3.91 -1.34,7.83 -1.82,11.77 -0.51,3.94 -0.89,7.89 -1.15,11.85 -0.44,7.92 -0.72,15.88 -0.31,23.81 0.37,7.92 1.49,15.82 3.42,23.52 0.95,3.85 2.12,7.65 3.46,11.39 1.33,3.74 2.84,7.42 4.54,11.01 -1.75,-3.56 -3.32,-7.22 -4.71,-10.95 -1.37,-3.73 -2.55,-7.53 -3.53,-11.39 -1.98,-7.7 -3.15,-15.62 -3.55,-23.56 -0.37,-7.95 -0.05,-15.91 0.44,-23.84 0.28,-3.96 0.68,-7.92 1.21,-11.86 0.55,-3.94 1.22,-7.85 2,-11.75
      z

      m 34.79,90.44 1.39,-8.17
      c 0.21,-1.37 0.35,-2.74 0.52,-4.11
      l 0.5,-4.11 0.03,-0.24 0.14,0.2
      c 0.98,1.37 1.47,3.03 1.62,4.69 0.18,1.66 0.03,3.34 -0.24,4.98 -0.28,1.64 -0.74,3.24 -1.28,4.81 -0.54,1.57 -1.18,3.1 -1.92,4.59
      l -0.03,-0.01
      c 0.67,-1.51 1.27,-3.06 1.78,-4.63 0.51,-1.57 0.92,-3.17 1.2,-4.8 0.28,-1.62 0.4,-3.28 0.25,-4.91 -0.13,-1.63 -0.59,-3.26 -1.53,-4.61
      l 0.17,-0.05 -0.53,4.11
      c -0.19,1.37 -0.33,2.74 -0.56,4.11
      l -1.48,8.16
      h -0.03
      z

      m -10.5,1.34
      c 0.74,-3.01 1.28,-6.07 1.65,-9.15 0.38,-3.08 0.57,-6.18 0.52,-9.28 -0.06,-3.09 -0.37,-6.2 -1.13,-9.2 -0.76,-2.99 -2.01,-5.91 -3.98,-8.28
      l -0.03,-0.03 0.01,-0.04 0.03,-0.14 0.02,-0.07
      v -0.02
      c 0,0 0,-0.01 0,-0.01 -0.11,-0.1 0.31,0 0.2,0.04
      v 0.03
      c 0,0 -0.06,0.28 -0.06,0.28
      l -0.11,0.56 -0.22,1.11 -0.44,2.22
      c -0.14,0.74 -0.3,1.48 -0.49,2.21
      l -0.55,2.2
      c -0.75,2.92 -1.35,5.88 -1.87,8.85 -0.27,1.49 -0.48,2.97 -0.54,4.48 -0.07,1.5 -0.03,3.01 0.09,4.52 0.25,3 0.82,5.98 1.59,8.9
      H 45.12
      c -0.84,-2.89 -1.46,-5.87 -1.73,-8.88 -0.14,-1.51 -0.19,-3.02 -0.14,-4.54 0.05,-1.52 0.25,-3.02 0.5,-4.51 0.51,-2.98 1.12,-5.94 1.87,-8.87
      l 0.55,-2.19
      c 0.2,-0.73 0.36,-1.46 0.5,-2.2
      l 0.45,-2.22 0.22,-1.11 0.11,-0.56 0.05,-0.28
      v -0.03
      c -0.1,0.04 0.32,0.13 0.21,0.04
      v 0
      c 0,0 0,0.03 0,0.03
      l -0.02,0.07 -0.03,0.14 -0.02,-0.09
      c 1.99,2.42 3.23,5.36 3.98,8.37 0.75,3.02 1.05,6.14 1.1,9.24 0.04,3.1 -0.18,6.21 -0.57,9.29 -0.39,3.08 -0.95,6.14 -1.7,9.15
      h -0.03
      z

      M 30.19,162.32
      c -0.44,0.67 -0.78,1.4 -1.02,2.16 -0.23,0.77 -0.4,1.55 -0.48,2.34 -0.17,1.59 -0.07,3.19 0.17,4.77 0.48,3.16 1.48,6.22 2.58,9.22 0.55,1.5 1.15,2.99 1.74,4.48 0.58,1.49 1.2,2.97 1.85,4.43 1.27,2.94 2.72,5.79 4.26,8.6
      l -0.02,0.02
      c -1.67,-2.73 -3.12,-5.6 -4.47,-8.51 -1.31,-2.93 -2.45,-5.92 -3.54,-8.94 -1.08,-3.01 -2.06,-6.09 -2.51,-9.27 -0.22,-1.59 -0.31,-3.2 -0.12,-4.8 0.09,-0.8 0.27,-1.58 0.51,-2.35 0.26,-0.76 0.6,-1.49 1.05,-2.16
      l 0.02,0.02
      z

      m 1.98,-2.92
      c 0.83,-0.04 1.65,0.27 2.28,0.8 0.64,0.52 1.13,1.2 1.53,1.92 0.8,1.44 1.3,3.02 1.71,4.61 0.8,3.18 1.23,6.44 1.62,9.69 0.19,1.63 0.37,3.25 0.54,4.89 0.16,1.63 0.29,3.26 0.38,4.9 0.16,3.27 0.24,6.55 0.14,9.83
      h -0.03
      c -0.05,-3.27 -0.13,-6.55 -0.36,-9.81 -0.19,-3.26 -0.53,-6.52 -0.87,-9.78 -0.37,-3.25 -0.77,-6.51 -1.54,-9.69 -0.4,-1.58 -0.87,-3.16 -1.65,-4.6 -0.39,-0.72 -0.87,-1.39 -1.5,-1.91 -0.62,-0.53 -1.43,-0.85 -2.24,-0.81
      v -0.03
      z

      m 27.38,18.71
      c 0.19,3.38 -0.02,6.79 -0.65,10.12 -0.63,3.33 -1.7,6.59 -3.28,9.6
      l -0.22,0.41 0.02,-0.46 0.23,-4.81 0.07,-1.2 0.11,-1.2
      c 0.07,-0.8 0.16,-1.6 0.26,-2.4 0.41,-3.18 0.94,-6.36 1.69,-9.48
      h 0.03
      c -0.61,3.15 -1.13,6.32 -1.47,9.51 -0.17,1.59 -0.36,3.18 -0.42,4.79
      l -0.28,4.8 -0.2,-0.05
      c 1.59,-2.98 2.68,-6.21 3.34,-9.53 0.66,-3.31 0.9,-6.71 0.75,-10.1
      h 0.03
      z

      M 18.3,129.59
      c -1.78,-3.66 -2.66,-7.68 -3.16,-11.71 -0.46,-4.04 -0.48,-8.11 -0.29,-12.17 0.42,-8.12 1.73,-16.16 3.39,-24.1 1.72,-7.93 3.86,-15.77 6.24,-23.53 1.2,-3.88 2.45,-7.73 3.78,-11.57 1.34,-3.83 2.74,-7.64 4.2,-11.42 -1.41,3.8 -2.75,7.63 -4.03,11.48 -1.3,3.84 -2.54,7.7 -3.71,11.58 -2.35,7.76 -4.44,15.6 -6.12,23.54 -1.7,7.92 -3.05,15.94 -3.51,24.03 -0.22,4.04 -0.21,8.11 0.22,12.14 0.44,4.01 1.25,8.04 2.98,11.73
      z

      m 116.23,-0.2
      c 1.69,-3.7 2.45,-7.75 2.84,-11.77 0.38,-4.04 0.34,-8.1 0.08,-12.15 -0.55,-8.09 -1.99,-16.11 -3.78,-24.02 -1.77,-7.92 -3.95,-15.74 -6.38,-23.49 -1.22,-3.87 -2.5,-7.72 -3.84,-11.55 -1.33,-3.83 -2.71,-7.65 -4.16,-11.45 1.51,3.77 2.95,7.57 4.33,11.38 1.37,3.82 2.67,7.67 3.91,11.54 2.47,7.73 4.7,15.56 6.51,23.48 1.75,7.93 3.16,15.96 3.66,24.08 0.24,4.05 0.26,8.13 -0.15,12.18 -0.46,4.03 -1.28,8.07 -3.03,11.75
      z"
    />
    <path
      key="outline"
      {...regular(stroke)}
      d="m 71.86,16.14 2.85,0.25 -2.85,0.25
      z

      m -0.18,0.37
      a 1.65,1.65 0 0 1 -1.65,1.65 1.65,1.65 0 0 1 -1.65,-1.65 1.65,1.65 0 0 1 1.65,-1.65 1.65,1.65 0 0 1 1.65,1.65
      z

      M 65.89,10.62
      v 11.49

      m 57.54,16.96
      c -6.53,-2.35 -13.92,-5.43 -20.27,-9.29 -3.63,-2.21 -6.25,-3.7 -10.05,-7.83

      M 76.08,92.19
      V 22.11

      M 86.09,199.49
      h 37.32
      l -1.05,13.1
      H 88.65
      C 86,173.38 79.02,99.18 78.31,96.45 77.85,94.66 76.98,93.32 76.31,92.44
      H 75.85
      c -0.68,0.88 -1.54,2.22 -2,4
      C 73.13,99.18 66.15,173.38 63.51,212.59
      H 29.79
      l -1.05,-13.1
      h 37.32

      M 116.35,18.51
      c -0.95,0.27 -2.93,0.67 -4.76,1.1 -1.75,0.41 -9.13,2.06 -22.53,2.5 -3.12,0.1 -7.4,0.17 -12.51,0
      h -0.96
      c -5.12,0.17 -9.39,0.1 -12.51,0
      C 49.69,21.67 42.31,20.01 40.55,19.61 38.73,19.18 36.74,18.78 35.8,18.51

      M 115.85,7.34
      c -1.01,0.35 -2.47,0.81 -4.26,1.25 -3.92,0.98 -7.15,1.3 -10.01,1.5 -9.73,0.68 -24.31,0.51 -25.03,0.5
      H 75.6
      C 74.88,10.61 60.3,10.77 50.57,10.1 47.71,9.9 44.47,9.57 40.55,8.59 38.77,8.15 37.31,7.69 36.3,7.34

      M 123.36,199.57
      c 0.16,-1.16 0.94,-4.92 1.5,-7.01 1.68,-6.23 8.37,-20.24 18.02,-40.55 0.79,-1.66 4.17,-8.75 4.51,-16.02 0.09,-1.94 0,-4 0,-4 -0.21,-4.97 -1.15,-8.62 -1.5,-10.01 -2.08,-8.16 -3.7,-16.43 -6.01,-24.53 -4.02,-14.11 -2.36,-9.27 -7.51,-27.53 -3.89,-13.82 -6.53,-23.13 -10.51,-35.54 -2.24,-6.97 -4.18,-12.68 -5.51,-16.52
      L 115.85,7.34
      c -0.85,-0.13 -2.05,-0.31 -3.5,-0.5 0,0 -4.41,-0.59 -9.51,-1
      C 98.33,5.48 89.43,5.01 77.81,4.84
      H 74.34
      C 62.72,5.01 53.82,5.48 49.31,5.84 44.21,6.26 39.8,6.84 39.8,6.84 38.35,7.03 37.15,7.21 36.3,7.34
      L 35.8,17.86
      C 34.47,21.7 32.53,27.41 30.29,34.38 26.31,46.78 23.67,56.1 19.78,69.92 14.63,88.18 16.3,83.34 12.27,97.45 9.96,105.54 8.34,113.82 6.26,121.98
      c -0.36,1.39 -1.29,5.04 -1.5,10.01 0,0 -0.09,2.07 0,4 0.33,7.27 3.71,14.36 4.51,16.02 9.65,20.3 16.34,34.32 18.02,40.55 0.56,2.09 1.35,5.85 1.5,7.01

      M 28.75,39.24
      c 6.53,-2.35 13.92,-5.43 20.27,-9.29 3.63,-2.21 6.25,-3.7 10.05,-7.83"
    />
  </>
)

/*
 * SVG elements for the back
 */
const Back = ({ stroke }) => (
  <>
    <path
      key="outline"
      {...regular(stroke)}
      d="m 284.79,212.64 -6.51,-6.32 6.22,-6.82

      m -82.83,13.41 6.51,-6.1 -6.11,-7.35

      M 243.27,92.19
      V 22.11

      M 233.26,199.49
      h -37.32
      l 1.05,13.1
      h 33.71
      c 2.65,-39.21 9.63,-113.4 10.34,-116.14 0.46,-1.79 1.33,-3.13 2,-4
      h 0.46
      c 0.68,0.88 1.54,2.22 2,4 0.71,2.73 7.69,76.93 10.34,116.14
      h 33.71
      l 1.05,-13.1
      H 253.3

      M 203,18.51
      c 0.95,0.27 2.93,0.67 4.76,1.1 1.75,0.41 9.13,2.06 22.53,2.5 3.12,0.1 7.4,0.17 12.51,0
      h 0.96
      c 5.12,0.17 9.39,0.1 12.51,0 13.4,-0.44 20.77,-2.1 22.53,-2.5 1.83,-0.43 3.81,-0.83 4.76,-1.1

      M 203.5,7.34
      c 1.01,0.35 2.47,0.81 4.26,1.25 3.92,0.98 7.15,1.3 10.01,1.5 9.73,0.68 24.31,0.51 25.03,0.5
      h 0.96
      c 0.72,0.01 15.3,0.18 25.03,-0.5 2.86,-0.2 6.09,-0.53 10.01,-1.5 1.78,-0.45 3.24,-0.91 4.26,-1.25

      M 195.99,199.57
      c -0.16,-1.16 -0.94,-4.92 -1.5,-7.01 -1.68,-6.23 -8.37,-20.24 -18.02,-40.55 -0.79,-1.66 -4.17,-8.75 -4.51,-16.02 -0.09,-1.94 0,-4 0,-4 0.21,-4.97 1.15,-8.62 1.5,-10.01 2.08,-8.16 3.7,-16.43 6.01,-24.53 4.02,-14.11 2.36,-9.27 7.51,-27.53 3.89,-13.82 6.53,-23.13 10.51,-35.54 2.24,-6.97 4.18,-12.68 5.51,-16.52
      l 0.5,-10.51
      c 0.85,-0.13 2.05,-0.31 3.5,-0.5 0,0 4.41,-0.59 9.51,-1 4.51,-0.37 13.41,-0.84 25.03,-1
      h 3.46
      c 11.62,0.17 20.52,0.64 25.03,1 5.1,0.42 9.51,1 9.51,1 1.45,0.19 2.66,0.37 3.5,0.5
      l 0.5,10.51
      c 1.33,3.84 3.27,9.55 5.51,16.52 3.98,12.41 6.62,21.73 10.51,35.54 5.15,18.27 3.48,13.42 7.51,27.53 2.31,8.09 3.93,16.37 6.01,24.53 0.36,1.39 1.29,5.04 1.5,10.01 0,0 0.09,2.07 0,4 -0.33,7.27 -3.71,14.36 -4.51,16.02 -9.65,20.3 -16.34,34.32 -18.02,40.55 -0.56,2.09 -1.35,5.85 -1.5,7.01

      m -1.7,6.82 -2.85,-0.25
      v 0.5
      z

      m -6.33,0.13
      a 1.65,1.65 0 0 0 1.65,1.65 1.65,1.65 0 0 0 1.65,-1.65 1.65,1.65 0 0 0 -1.65,-1.65 1.65,1.65 0 0 0 -1.65,1.65
      z

      m -84.6,-0.13 2.85,0.25
      v -0.5
      z

      m 3.02,0.13
      a 1.65,1.65 0 0 0 1.65,1.65 1.65,1.65 0 0 0 1.65,-1.65 1.65,1.65 0 0 0 -1.65,-1.65 1.65,1.65 0 0 0 -1.65,1.65
      z"
    />
    <path
      key="folds"
      {...regular(stroke)}
      opacity={0.3}
      d="m 292.71,105.8
      c 0.78,3.89 1.46,7.81 2,11.75 0.53,3.94 0.93,7.89 1.21,11.86 0.49,7.93 0.81,15.89 0.44,23.84 -0.41,7.94 -1.58,15.86 -3.55,23.56 -0.97,3.85 -2.16,7.66 -3.53,11.39 -1.39,3.72 -2.96,7.38 -4.71,10.95 1.7,-3.59 3.21,-7.27 4.54,-11.01 1.34,-3.73 2.51,-7.53 3.46,-11.39 1.93,-7.7 3.05,-15.59 3.42,-23.52 0.42,-7.92 0.14,-15.88 -0.31,-23.81 -0.26,-3.96 -0.64,-7.91 -1.15,-11.85 -0.49,-3.94 -1.1,-7.87 -1.82,-11.77
      z

      m -34.81,90.44 -1.48,-8.16
      c -0.22,-1.36 -0.37,-2.74 -0.56,-4.11
      l -0.53,-4.11 0.17,0.05
      c -0.95,1.34 -1.41,2.97 -1.53,4.61 -0.15,1.64 -0.03,3.29 0.25,4.91 0.28,1.62 0.68,3.23 1.2,4.8 0.51,1.57 1.11,3.11 1.78,4.63
      l -0.03,0.01
      c -0.74,-1.48 -1.38,-3.01 -1.92,-4.59 -0.54,-1.57 -1.01,-3.17 -1.28,-4.81 -0.28,-1.64 -0.41,-3.31 -0.24,-4.98 0.15,-1.66 0.64,-3.32 1.62,-4.69
      l 0.14,-0.2 0.03,0.24 0.5,4.11
      c 0.18,1.37 0.31,2.74 0.52,4.11
      l 1.39,8.17
      h -0.03
      z

      m 10.5,1.34
      c -0.75,-3.01 -1.31,-6.07 -1.7,-9.15 -0.39,-3.08 -0.61,-6.18 -0.57,-9.29 0.05,-3.1 0.35,-6.22 1.1,-9.24 0.75,-3.01 1.99,-5.95 3.98,-8.37
      l -0.02,0.09 -0.03,-0.14 -0.02,-0.07
      v -0.02
      c 0,0 0,-0 0,-0 -0.12,0.09 0.31,0 0.2,-0.04
      v 0.03
      c 0,0 0.06,0.28 0.06,0.28
      l 0.11,0.56 0.22,1.11 0.45,2.22
      c 0.15,0.74 0.31,1.48 0.5,2.2
      l 0.55,2.19
      c 0.75,2.93 1.36,5.89 1.87,8.87 0.25,1.49 0.45,3 0.5,4.51 0.05,1.51 0,3.03 -0.14,4.54 -0.27,3.01 -0.89,5.99 -1.73,8.89
      h -0.03
      c 0.77,-2.92 1.35,-5.9 1.59,-8.9 0.13,-1.5 0.15,-3.01 0.09,-4.52 -0.07,-1.51 -0.28,-2.99 -0.54,-4.48 -0.52,-2.97 -1.12,-5.93 -1.87,-8.85
      l -0.55,-2.2
      c -0.19,-0.73 -0.35,-1.47 -0.49,-2.21
      l -0.44,-2.22 -0.22,-1.11 -0.11,-0.56 -0.06,-0.28
      v -0.03
      c -0.11,-0.05 0.31,-0.14 0.2,-0.05
      v 0.01
      c 0,0 0,0.02 0,0.02
      l 0.02,0.07 0.03,0.14 0.01,0.05 -0.03,0.03
      c -1.98,2.38 -3.22,5.29 -3.98,8.28 -0.76,3 -1.07,6.1 -1.13,9.2 -0.06,3.1 0.14,6.2 0.52,9.28 0.38,3.08 0.92,6.14 1.65,9.15
      h -0.03
      z

      m 20.3,-35.69
      c 0.45,0.67 0.79,1.4 1.05,2.16 0.24,0.77 0.42,1.55 0.51,2.35 0.19,1.6 0.1,3.21 -0.12,4.8 -0.46,3.18 -1.44,6.25 -2.51,9.27 -1.1,3.01 -2.24,6 -3.54,8.94 -1.34,2.91 -2.79,5.77 -4.47,8.51
      l -0.02,-0.02
      c 1.54,-2.8 2.99,-5.66 4.26,-8.6 0.65,-1.46 1.27,-2.94 1.85,-4.43 0.59,-1.49 1.19,-2.97 1.74,-4.48 1.1,-3 2.1,-6.06 2.58,-9.22 0.24,-1.58 0.34,-3.18 0.17,-4.77 -0.09,-0.8 -0.26,-1.58 -0.48,-2.34 -0.25,-0.76 -0.59,-1.49 -1.02,-2.16
      l 0.02,-0.02
      z

      m -2.01,-2.89
      c -0.82,-0.03 -1.63,0.28 -2.24,0.81 -0.63,0.52 -1.11,1.2 -1.5,1.91 -0.78,1.44 -1.26,3.01 -1.65,4.6 -0.77,3.18 -1.18,6.43 -1.54,9.69 -0.34,3.25 -0.68,6.51 -0.87,9.78 -0.23,3.26 -0.32,6.54 -0.36,9.81
      h -0.03
      c -0.11,-3.27 -0.02,-6.55 0.14,-9.83 0.1,-1.64 0.23,-3.27 0.38,-4.9 0.17,-1.63 0.35,-3.25 0.54,-4.89 0.39,-3.25 0.82,-6.51 1.62,-9.69 0.41,-1.58 0.91,-3.16 1.71,-4.61 0.4,-0.72 0.89,-1.4 1.53,-1.92 0.63,-0.53 1.45,-0.84 2.28,-0.8
      v 0.03
      z

      m -27.36,18.69
      c -0.16,3.38 0.09,6.78 0.75,10.1 0.66,3.31 1.74,6.55 3.34,9.53
      l -0.2,0.05 -0.28,-4.8
      c -0.06,-1.6 -0.25,-3.19 -0.42,-4.79 -0.34,-3.19 -0.86,-6.35 -1.47,-9.5
      h 0.03
      c 0.75,3.12 1.28,6.29 1.69,9.48 0.1,0.8 0.18,1.6 0.26,2.4
      l 0.11,1.2 0.07,1.2 0.23,4.81 0.02,0.46 -0.22,-0.41
      c -1.58,-3.01 -2.65,-6.27 -3.28,-9.6 -0.63,-3.33 -0.84,-6.74 -0.65,-10.12
      z

      m 41.23,-48.52
      c 1.73,-3.68 2.54,-7.71 2.98,-11.73 0.43,-4.03 0.44,-8.09 0.22,-12.14 -0.46,-8.09 -1.81,-16.11 -3.51,-24.03 -1.68,-7.93 -3.77,-15.77 -6.12,-23.54 -1.18,-3.88 -2.41,-7.74 -3.71,-11.58 -1.28,-3.85 -2.62,-7.67 -4.03,-11.48 1.46,3.78 2.86,7.59 4.2,11.42 1.33,3.83 2.58,7.69 3.78,11.57 2.39,7.75 4.53,15.59 6.24,23.53 1.66,7.94 2.98,15.98 3.39,24.1 0.2,4.05 0.17,8.13 -0.29,12.17 -0.5,4.02 -1.37,8.05 -3.16,11.71
      z

      M 194,105.8
      c -0.72,3.91 -1.34,7.83 -1.82,11.77 -0.51,3.94 -0.89,7.89 -1.15,11.85 -0.44,7.92 -0.72,15.88 -0.31,23.81 0.37,7.92 1.49,15.82 3.42,23.52 0.95,3.85 2.12,7.65 3.46,11.39 1.33,3.74 2.84,7.42 4.54,11.01 -1.75,-3.56 -3.32,-7.22 -4.71,-10.95 -1.37,-3.73 -2.55,-7.53 -3.53,-11.39 -1.98,-7.7 -3.15,-15.62 -3.55,-23.56 -0.37,-7.95 -0.05,-15.91 0.44,-23.84 0.28,-3.96 0.68,-7.92 1.21,-11.86 0.55,-3.94 1.22,-7.85 2,-11.75
      z

      m 34.79,90.44 1.39,-8.17
      c 0.21,-1.37 0.35,-2.74 0.52,-4.11
      l 0.5,-4.11 0.03,-0.24 0.14,0.2
      c 0.98,1.37 1.47,3.03 1.62,4.69 0.18,1.66 0.03,3.34 -0.24,4.98 -0.28,1.64 -0.74,3.24 -1.28,4.81 -0.54,1.57 -1.18,3.1 -1.92,4.59
      l -0.03,-0.01
      c 0.67,-1.51 1.27,-3.06 1.78,-4.63 0.51,-1.57 0.92,-3.17 1.2,-4.8 0.28,-1.62 0.4,-3.28 0.25,-4.91 -0.13,-1.63 -0.59,-3.26 -1.53,-4.61
      l 0.17,-0.05 -0.53,4.11
      c -0.19,1.37 -0.34,2.74 -0.56,4.11
      l -1.48,8.16
      h -0.03
      z

      m -10.5,1.34
      c 0.74,-3.01 1.28,-6.07 1.65,-9.15 0.38,-3.08 0.57,-6.18 0.52,-9.28 -0.06,-3.09 -0.37,-6.2 -1.13,-9.2 -0.76,-2.99 -2.01,-5.91 -3.98,-8.28
      l -0.03,-0.03 0.01,-0.04 0.03,-0.14 0.02,-0.07
      v -0.02
      c 0,0 0,-0.01 0,-0.01 -0.11,-0.1 0.31,0 0.2,0.04
      v 0.03
      c 0,0 -0.06,0.28 -0.06,0.28
      l -0.11,0.56 -0.22,1.11 -0.44,2.22
      c -0.14,0.74 -0.3,1.48 -0.49,2.21
      l -0.55,2.2
      c -0.75,2.92 -1.35,5.88 -1.87,8.85 -0.27,1.49 -0.48,2.97 -0.54,4.48 -0.07,1.5 -0.03,3.01 0.09,4.52 0.25,3 0.82,5.98 1.59,8.9
      h -0.03
      c -0.84,-2.89 -1.46,-5.87 -1.73,-8.88 -0.14,-1.51 -0.19,-3.02 -0.14,-4.54 0.05,-1.52 0.25,-3.02 0.5,-4.51 0.51,-2.98 1.12,-5.94 1.87,-8.87
      l 0.55,-2.19
      c 0.2,-0.73 0.36,-1.46 0.5,-2.2
      l 0.45,-2.22 0.22,-1.11 0.11,-0.56 0.06,-0.28
      v -0.03
      c -0.1,0.04 0.32,0.13 0.21,0.04
      v 0
      c 0,0 0,0.03 0,0.03
      l -0.02,0.07 -0.03,0.14 -0.02,-0.09
      c 1.99,2.42 3.23,5.36 3.98,8.37 0.75,3.02 1.05,6.14 1.1,9.24 0.04,3.1 -0.18,6.21 -0.57,9.29 -0.39,3.08 -0.95,6.14 -1.7,9.15
      h -0.03
      z

      m -20.25,-35.67
      c -0.44,0.67 -0.78,1.4 -1.02,2.16 -0.23,0.77 -0.4,1.55 -0.48,2.34 -0.17,1.59 -0.07,3.19 0.17,4.77 0.48,3.16 1.48,6.22 2.58,9.22 0.55,1.5 1.15,2.99 1.74,4.48 0.58,1.49 1.2,2.97 1.85,4.43 1.27,2.94 2.72,5.79 4.26,8.6
      l -0.02,0.02
      c -1.67,-2.73 -3.12,-5.6 -4.47,-8.51 -1.31,-2.93 -2.45,-5.92 -3.54,-8.94 -1.08,-3.01 -2.06,-6.09 -2.51,-9.27 -0.22,-1.59 -0.31,-3.2 -0.12,-4.8 0.09,-0.8 0.27,-1.58 0.51,-2.35 0.26,-0.76 0.6,-1.49 1.05,-2.16
      l 0.02,0.02
      z

      m 1.99,-2.92
      c 0.83,-0.04 1.65,0.27 2.28,0.8 0.64,0.52 1.13,1.2 1.53,1.92 0.8,1.44 1.3,3.02 1.71,4.61 0.8,3.18 1.23,6.44 1.62,9.69 0.19,1.63 0.37,3.25 0.54,4.89 0.16,1.63 0.29,3.26 0.38,4.9 0.16,3.27 0.24,6.55 0.14,9.83
      h -0.03
      c -0.05,-3.27 -0.13,-6.55 -0.36,-9.81 -0.19,-3.26 -0.53,-6.52 -0.87,-9.78 -0.37,-3.25 -0.77,-6.51 -1.54,-9.69 -0.4,-1.58 -0.87,-3.16 -1.65,-4.6 -0.39,-0.72 -0.87,-1.39 -1.5,-1.91 -0.62,-0.53 -1.43,-0.85 -2.24,-0.81
      v -0.03
      z

      m 27.38,18.71
      c 0.19,3.38 -0.02,6.79 -0.65,10.12 -0.63,3.33 -1.7,6.59 -3.28,9.6
      l -0.22,0.41 0.02,-0.46 0.23,-4.81 0.07,-1.2 0.11,-1.2
      c 0.07,-0.8 0.16,-1.6 0.26,-2.4 0.41,-3.18 0.94,-6.36 1.69,-9.48
      h 0.03
      c -0.61,3.15 -1.13,6.32 -1.47,9.51 -0.17,1.59 -0.36,3.18 -0.42,4.79
      l -0.28,4.8 -0.2,-0.05
      c 1.59,-2.98 2.68,-6.21 3.34,-9.53 0.66,-3.31 0.9,-6.71 0.75,-10.1
      h 0.03
      z

      m -41.26,-48.52
      c -1.78,-3.66 -2.66,-7.68 -3.16,-11.71 -0.46,-4.04 -0.48,-8.11 -0.29,-12.17 0.42,-8.12 1.73,-16.16 3.39,-24.1 1.72,-7.93 3.86,-15.77 6.24,-23.53 1.2,-3.88 2.45,-7.73 3.78,-11.57 1.34,-3.83 2.74,-7.64 4.2,-11.42 -1.41,3.8 -2.75,7.63 -4.03,11.48 -1.3,3.84 -2.54,7.7 -3.71,11.58 -2.35,7.76 -4.44,15.6 -6.12,23.54 -1.7,7.92 -3.05,15.94 -3.51,24.03 -0.22,4.04 -0.21,8.11 0.22,12.14 0.44,4.01 1.25,8.04 2.98,11.73
      z"
    />
  </>
)
